"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.command = void 0;
const path = require("path");
const command_1 = require("../command");
const dataconnectEmulator_1 = require("../emulator/dataconnectEmulator");
const projectUtils_1 = require("../projectUtils");
const load_1 = require("../dataconnect/load");
const fileUtils_1 = require("../dataconnect/fileUtils");
const logger_1 = require("../logger");
exports.command = new command_1.Command("dataconnect:sdk:generate")
    .description("generates typed SDKs for your Data Connect connectors")
    .action(async (options) => {
    const projectId = (0, projectUtils_1.needProjectId)(options);
    const services = (0, fileUtils_1.readFirebaseJson)(options.config);
    for (const service of services) {
        let configDir = service.source;
        if (!path.isAbsolute(configDir)) {
            const cwd = options.cwd || process.cwd();
            configDir = path.resolve(path.join(cwd), configDir);
        }
        const serviceInfo = await (0, load_1.load)(projectId, service.location, configDir);
        for (const conn of serviceInfo.connectorInfo) {
            const output = await dataconnectEmulator_1.DataConnectEmulator.generate({
                configDir,
                locationId: service.location,
                connectorId: conn.connectorYaml.connectorId,
            });
            logger_1.logger.info(output);
            logger_1.logger.info(`Generated SDKs for ${conn.connectorYaml.connectorId}`);
        }
    }
});
